"""
AuroraPay
"""

from .client import AuroraPayClient
from .exceptions import (
    AuroraPayError,
    ValidationError,
    OrderCreationError,
    OrderNotFoundError,
    UnauthorizedError,
    InactiveShopError,
    ForbiddenError,
)
from .models import (
    PaymentMethod,
    PaymentStatus,
    CreateOrderRequest,
    CreateOrderResponse,
    PaymentStatusResponse,
)

__version__ = "1.0.0"
__all__ = [
    "AuroraPayClient",
    "AuroraPayError",
    "ValidationError",
    "OrderCreationError",
    "OrderNotFoundError",
    "UnauthorizedError",
    "InactiveShopError",
    "ForbiddenError",
    "PaymentMethod",
    "PaymentStatus",
    "CreateOrderRequest",
    "CreateOrderResponse",
    "PaymentStatusResponse",
]



"""
Модели данных для AuroraPay Payment API
"""

from enum import Enum
from typing import Optional
from dataclasses import dataclass


class PaymentMethod(str, Enum):
    """Методы оплаты"""
    CARD = "card"  # Карты РФ
    SBP = "sbp"    # СБП (Система Быстрых Платежей)
    KZT = "kzt"    # Карты Казахстана


class PaymentStatus(str, Enum):
    """Статусы платежа"""
    PENDING = "pending"           # Ожидает обработки
    PROCESSING = "processing"     # В процессе обработки
    COMPLETED = "completed"       # Успешно завершен
    FAILED = "failed"            # Ошибка при обработке
    CANCELED = "canceled"        # Отменен пользователем
    AUTOCANCELED = "autocanceled"  # Автоматически отменен


@dataclass
class CreateOrderRequest:
    """Запрос на создание заказа"""
    amount: float
    payment_method: PaymentMethod
    
    def to_dict(self) -> dict:
        return {
            "amount": self.amount,
            "paymentMethod": self.payment_method.value
        }


@dataclass
class PaymentDetails:
    """Детали платежа"""
    bank_name: Optional[str]
    cardholder: Optional[str]
    wallet_payment: str
    order_status: str
    amount_payable: str
    amount_receivable: str
    exchange_rate: str


@dataclass
class CreateOrderResponse:
    """Ответ на создание заказа"""
    order_id: str
    bank: str
    amount: float
    commission: float
    commission_percent: float
    payment_details: PaymentDetails
    payment_link: str
    
    @classmethod
    def from_dict(cls, data: dict) -> "CreateOrderResponse":
        payment_details = PaymentDetails(
            bank_name=data["paymentDetails"].get("bankName"),
            cardholder=data["paymentDetails"].get("cardholder"),
            wallet_payment=data["paymentDetails"]["walletPayment"],
            order_status=data["paymentDetails"]["orderStatus"],
            amount_payable=data["paymentDetails"]["amountPayable"],
            amount_receivable=data["paymentDetails"]["amountReceivable"],
            exchange_rate=data["paymentDetails"]["exchangeRate"],
        )
        
        return cls(
            order_id=data["orderId"],
            bank=data["bank"],
            amount=data["amount"],
            commission=data["commission"],
            commission_percent=data["commissionPercent"],
            payment_details=payment_details,
            payment_link=data["paymentLink"],
        )


@dataclass
class PaymentStatusResponse:
    """Ответ на запрос статуса платежа"""
    order_id: str
    status: PaymentStatus
    amount: float
    payment_method: Optional[str]
    payment_details: dict
    created_at: str
    updated_at: str
    
    @classmethod
    def from_dict(cls, data: dict) -> "PaymentStatusResponse":
        return cls(
            order_id=data["orderId"],
            status=PaymentStatus(data["status"]),
            amount=data["amount"],
            payment_method=data.get("paymentMethod"),
            payment_details=data["paymentDetails"],
            created_at=data["createdAt"],
            updated_at=data["updatedAt"],
        )



"""
Исключения для AuroraPay Payment API
"""


class AuroraPayError(Exception):
    """Базовое исключение для всех ошибок API"""
    
    def __init__(self, message: str, code: str = None, status_code: int = None):
        self.message = message
        self.code = code
        self.status_code = status_code
        super().__init__(self.message)


class ValidationError(AuroraPayError):
    """Ошибка валидации входных данных"""
    pass


class OrderCreationError(AuroraPayError):
    """Ошибка создания заказа"""
    pass


class OrderNotFoundError(AuroraPayError):
    """Заказ не найден"""
    pass


class UnauthorizedError(AuroraPayError):
    """Отсутствует или неверный API ключ"""
    pass


class InactiveShopError(AuroraPayError):
    """Магазин неактивен"""
    pass


class ForbiddenError(AuroraPayError):
    """Доступ запрещен"""
    pass



"""
Клиент для работы с AuroraPay Payment API
"""

import requests
from typing import Optional
from .models import CreateOrderRequest, CreateOrderResponse, PaymentStatusResponse
from .exceptions import (
    AuroraPayError,
    ValidationError,
    OrderCreationError,
    OrderNotFoundError,
    UnauthorizedError,
    InactiveShopError,
    ForbiddenError,
)


class AuroraPayClient:
    """
    Клиент для работы с AuroraPay Payment API
    
    Пример использования:
        client = AuroraPayClient(api_key="your_api_key_here")
        
        # Создать заказ
        order = client.create_order(amount=5000, payment_method="card")
        print(f"Order ID: {order.order_id}")
        print(f"Payment Link: {order.payment_link}")
        
        # Проверить статус
        status = client.get_payment_status(order.order_id)
        print(f"Status: {status.status}")
    """
    
    def __init__(
        self,
        api_key: str,
        base_url: str = "https://goiskra.com",
        timeout: int = 30
    ):
        """
        Инициализация клиента
        
        Args:
            api_key: API ключ для аутентификации
            base_url: Базовый URL API (по умолчанию: https://goiskra.com)
            timeout: Таймаут запросов в секундах (по умолчанию: 30)
        """
        self.api_key = api_key
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.session = requests.Session()
        self.session.headers.update({
            "X-API-Key": self.api_key,
            "Content-Type": "application/json",
        })
    
    def _handle_error(self, response: requests.Response) -> None:
        """Обработка ошибок API"""
        try:
            error_data = response.json()
            error = error_data.get("error", {})
            code = error.get("code", "UNKNOWN_ERROR")
            message = error.get("message", "Unknown error occurred")
        except Exception:
            code = "UNKNOWN_ERROR"
            message = f"HTTP {response.status_code}: {response.text}"
        
        # Маппинг кодов ошибок на исключения
        error_map = {
            "VALIDATION_ERROR": ValidationError,
            "ORDER_CREATION_FAILED": OrderCreationError,
            "ORDER_NOT_FOUND": OrderNotFoundError,
            "UNAUTHORIZED": UnauthorizedError,
            "INACTIVE_SHOP": InactiveShopError,
            "FORBIDDEN": ForbiddenError,
        }
        
        exception_class = error_map.get(code, AuroraPayError)
        raise exception_class(message, code=code, status_code=response.status_code)
    
    def create_order(
        self,
        amount: float,
        payment_method: str
    ) -> CreateOrderResponse:
        """
        Создать новый заказ на оплату
        
        Args:
            amount: Сумма платежа в рублях (минимум 500)
            payment_method: Метод оплаты ("card", "sbp", "kzt")
        
        Returns:
            CreateOrderResponse: Данные созданного заказа
        
        Raises:
            ValidationError: Ошибка валидации входных данных
            OrderCreationError: Ошибка создания заказа
            UnauthorizedError: Неверный API ключ
            InactiveShopError: Магазин неактивен
        
        Example:
            order = client.create_order(amount=5000, payment_method="card")
            print(f"Order ID: {order.order_id}")
            print(f"Payment Link: {order.payment_link}")
            print(f"Bank: {order.payment_details.bank_name}")
            print(f"Card/Phone: {order.payment_details.wallet_payment}")
        """
        url = f"{self.base_url}/api/order/create"
        payload = {
            "amount": amount,
            "paymentMethod": payment_method
        }
        
        try:
            response = self.session.post(url, json=payload, timeout=self.timeout)
            
            if response.status_code in (200, 201):
                data = response.json()
                return CreateOrderResponse.from_dict(data["data"])
            else:
                self._handle_error(response)
        except requests.exceptions.RequestException as e:
            raise AuroraPayError(f"Network error: {str(e)}")
    
    def get_payment_status(self, order_id: str) -> PaymentStatusResponse:
        """
        Получить статус платежа
        
        Args:
            order_id: ID заказа
        
        Returns:
            PaymentStatusResponse: Статус платежа
        
        Raises:
            OrderNotFoundError: Заказ не найден
            UnauthorizedError: Неверный API ключ
            ForbiddenError: Доступ запрещен
        
        Example:
            status = client.get_payment_status("AURO198329797418RA")
            print(f"Status: {status.status}")
            print(f"Amount: {status.amount}")
            print(f"Created: {status.created_at}")
        """
        url = f"{self.base_url}/api/payment/status/{order_id}"
        
        try:
            response = self.session.get(url, timeout=self.timeout)
            
            if response.status_code == 200:
                data = response.json()
                return PaymentStatusResponse.from_dict(data["data"])
            else:
                self._handle_error(response)
        except requests.exceptions.RequestException as e:
            raise AuroraPayError(f"Network error: {str(e)}")
    
    def close(self):
        """Закрыть сессию"""
        self.session.close()
    
    def __enter__(self):
        """Context manager support"""
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager support"""
        self.close()
